//########################################################################################
// Copyright (C) 2000, ToneThink.Soft  All Rights Reserved.
// 文件创建时间：2019-09-05
//   文件创建人：peng
// 文件功能描述：通过jssip 控件，实现基于浏览器的软电话
//参考文档 https://jssip.net/documentation/3.8.x/api
//
//     维护记录：本demo 基于jssip 3.8.0 修改
//2020.05.17 peng 创建基于手机app模块
//2021.04.03 手机版与pc版：ut_phone_svc.js,ut_phone.min.js,myPhone.scss 相同
//           index.js 略有不同
//           支持视频会议
//#########################################################################################
import myPhone from './ut_phone.min';
import EventsClass from './events-class';
import {emitter} from './external';

var myAudio_speeker = null;  //播放振铃，回铃等
let _phone_sound  = "/ut_phone";

var CallStatus = "00"; //00-未注册  01-空闲 02-摘机状态 03-振钤状态 04-回铃状态 05-通话  06 挂断
//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
//处理SoftCall UI相关部分
//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
var m_emit = null;
var config_sip = {};//ext, password, host_wss
let _register_last = 0 //上次注册时间

const myEvent_phone = new EventsClass();
  myEvent_phone.on('phone-event', function (event) {
		onReceivePhoneInfo(event.code,event.data,event.info,event.more)
 })

function init_soft_phone(html_this, cfg_sip, cfg_media) {
    m_emit = html_this;

	myAudio_speeker = cfg_media.speeker;// document.getElementById('audio_speeker');

	//let audio = window.document.createElement('audio');
	//audio.volume = 0.1; //默认声音0.1
    //let video = document.getElementById('video_remote');
	myPhone.register_callback(myEvent_phone,cfg_media.audio,cfg_media.video);
	myPhone.debug(0x1);  //1-default 0x10-sip  0x20-原生sip
	config_sip = cfg_sip;
	_phone_sound = cfg_sip.sound || _phone_sound;
	myPhone.InitPhone(config_sip.ext, config_sip.pwd, config_sip.host);
	_register_last = new Date();
	setTimeout(check_task_1s_timer, 30000); //30S 检查一次
}

//检查注册状态
function check_task_1s_timer() {
	//$("#timer").html(getDateTimer());
	if(CallStatus == "00"){
		myPhone.InitPhone(config_sip.ext, config_sip.pwd, config_sip.host);
		_register_last = new Date();
	}
	else if(CallStatus == "01"){
		var vNow = new Date();
		var vSeconds = parseInt(vNow - _register_last) / 1000;//时间差毫秒
	    //每半小时注册一次
		if(vSeconds> 1800){
			myPhone.register();
			_register_last = new Date();
		}
	}
	setTimeout(check_task_1s_timer, 30000);
	return 1;
}


//收到Phone传来的消息，将事件消息传给客户端
//nEventCode:
//  1：呼入状态发生变化，EventData=分机号，EventInfo=呼叫状态，EventInfo_ex=主叫号等信息
//  2：呼出状态发生变化，EventData=分机号，EventInfo=呼叫状态，EventInfo_ex=主叫号等信息
//  100：注册状态发生变化，EventData=分机号，EventInfo=工作状态
function onReceivePhoneInfo(EventCode, EventData, EventInfo, EventMore='') {
	console.info("EventCode=" + EventCode + ",EventData=" + EventData + ",EventInfo=" + EventInfo + ",EventMore=" + EventMore);
	onReceivePhoneInfo_app(EventCode, EventData, EventInfo, EventMore);
	//onReceivePhoneInfo_book(EventCode, EventData, EventInfo, EventMore)
}

function onReceivePhoneInfo_app(EventCode, EventData, EventInfo, EventMore) {
	switch (EventCode) {
		case 1: //呼入状态发生变化
			CallStatus = EventInfo;
			if (EventInfo == "01") {
				setCallStatus('Idle');
			} else if (EventInfo == "03") {
				setCallStatus('Ringing', EventMore);
				play("incoming-call2.ogg",5);
			} else if (EventInfo == "05") {
				setCallStatus('Connect');
				stop();
			} else if (EventInfo == "06") {
				setCallStatus('disConnect');
				play("busy.wav",1);
				CallStatus = "01";
			} else //if(EventInfo == "03")
				setCallStatus('Connect');
			break;

		case 2: //呼出状态发生变化
			CallStatus = EventInfo;
			if (EventInfo == "01") {
				setCallStatus('Idle');
			} else if (EventInfo == "02") {
				setCallStatus('Pickup');
			} else if (EventInfo == "04") {
				setCallStatus('Ringback');
				//play("ringback.wav",5);
			} else if (EventInfo == "05") {
				setCallStatus('Connect');
				stop();
			} else if (EventInfo == "06") {
				setCallStatus('disConnect');
				play("busy.wav",1);
				CallStatus = "01";
			} else if (EventInfo == "09") { //呼叫失败
				setCallStatus('Fail', EventMore);
				if (EventMore == "Canceled")
					stop();
				else if (EventMore == "Busy")
					play("busy.wav",1);
				else if (EventMore == "Unavailable")
					play("notfound.wav",1);
				else
					play("outgoing-call-rejected.wav");
				CallStatus = "01";
			} else //if(EventInfo == "03")
				setCallStatus('Connect');
			break;

		case 100: //注册状态发生变化
			if (EventInfo == "01") {
				CallStatus = EventInfo;
				//$("#registered").children("i").attr("class", "icon-wifi");
				//setResult("分机:" + EventData + " 就绪");
			} else {
				//$("#registered").children("i").attr("class", "icon-plane");
				//setResult("分机:" + EventData + " 不可用");
			}
			setCallStatus('Registered', EventInfo);
			break;
	}
}

function PlaceCall(Called,mode) {
	let nReturn = -1;
	if (parseInt(CallStatus) < 1) {
		//setResult("电话未能成功注册，请核查");
		//play("notfound.wav");
		return nReturn;
	}

	if (CallStatus == '01') {
		myPhone.MakeCall(Called,mode);
		nReturn = 1;
	} else if (CallStatus == '03') {
		stop();
		myPhone.AnswerCall(Called,3);
		nReturn = 2
	}
	else {
		myPhone.ReleaseCall();
		//onReceivePhoneInfo(2, "", "01")
		nReturn = 0;
	}
	return nReturn;
}

function AnswerCall(mode) {
	myPhone.AnswerCall('',mode);
 }

function ReleaseCall() {
   myPhone.ReleaseCall();
}

function SendDTMF(SendDTMF) {
	if (CallStatus == '05') {
		myPhone.SendDTMF(SendDTMF);
	}
}

//设置电话状态
function setCallStatus(status, info) {
    if(m_emit){
      let obj = {code: status,info: info}
      emitter.emit('sip-event', obj)
	}
}

/*
//显示系统时间
function getDateTimer() {
	var dtNow = new Date();
	var days = dtNow.getFullYear() + "年" + (dtNow.getMonth() + 1) + "月" + dtNow.getDate() + "日" + ' 星期' + '日一二三四五六'.charAt(
		dtNow.getDay()) + "<br />";
	var hours = dtNow.getHours();
	var minutes = dtNow.getMinutes();
	var seconds = dtNow.getSeconds();
	//var timeValue = days;
	var timeValue = "";
	var nHour = (hours > 12) ? hours - 12 : hours;
	timeValue += ((nHour < 10) ? "0" : "") + nHour;
	timeValue += ((minutes < 10) ? ":0" : ":") + minutes;
	timeValue += ((seconds < 10) ? ":0" : ":") + seconds;
	timeValue += (hours >= 12) ? " PM" : "AM";
	return timeValue;
}
*/
//Uncaught (in promise) DOMException: The play() request was interrupted by a call to pause().
//问题的关键所在是在执行了play()方法以后立即执行pause()方法。反之亦然
//https://developer.mozilla.org/en-US/docs/Web/HTML/Element/audio

//第一次振铃，会报错：
//Uncaught (in promise) DOMException: play() failed because the user didn't interact with the document first. https://goo.gl/xX8pDD
function stop() {
	var audio = myAudio_speeker;//document.getElementById('audio_speeker');
	if (audio == null) return;

	//audio.src = _phone_sound + "/sounds/silen.wav";
	let wav = audio.src;
	audio.pause();
	console.log("stop<" + wav + ">");
}

function play(wav,loop=1) {
	var audio = myAudio_speeker;//document.getElementById('audio_speeker');
	if (audio == null) return;

	console.log("play<" + wav + ">,loop="+loop);

	var start = 0; //定义循环的变量
	var times = loop; //定于循环的次数

	audio.src = _phone_sound + "/sounds/" + wav;
	audio.play(); //启动音频，用于第一次启动
	//if(loop<2) return;
	audio.onended = function() {
		start++; //循环
		if (start < times){
			audio.play(); //启动音频，也就是播放
		}
		else{
			audio.pause(); //也就是当循环的变量等于次数的时候，就会终止循环并且关掉音频
		}
		//console.log("play<" + wav + "> onended,times="+ start);
	}
}

function register() {
	if(CallStatus == "00"){
		var vNow = new Date();
		var vSeconds = parseInt(vNow - _register_last) / 1000;//时间差毫秒
	    //至少间隔10秒注册一次
		if(vSeconds> 10){
			myPhone.register();
			_register_last = new Date();
		}
	}
}

function unRegister() {
	return myPhone.unRegister();
}


//改变本地视频状态 mode = 1 启用 0 不启用
function Mute(mode) {
	let options = {audio:true,video:true}
	if(mode==0) options = {audio:false,video:false};
	else if(mode==1) options = {audio:true,video:false};
	else if(mode==10) options = {audio:false,video:true};
	else if(mode==11) options = {audio:true,video:true};
	myPhone.Mute(options);
}
function unMute(mode) {
	let options = {audio:true,video:true}
	if(mode==0) options = {audio:false,video:false};
	else if(mode==1) options = {audio:true,video:false};
	else if(mode==10) options = {audio:false,video:true};
	else if(mode==11) options = {audio:true,video:true};
	myPhone.unMute(options);
}

export {
	init_soft_phone,
	register,
	unRegister,
	play,
	onReceivePhoneInfo,
	PlaceCall,
	AnswerCall,
	ReleaseCall,
	SendDTMF,
	Mute,
	unMute
}
